﻿using System;
using System.Threading.Tasks;
using System.Windows.Input;

namespace SfCalendarsample
{
    public sealed class AsyncCommand : ICommand
    {
        public event EventHandler CanExecuteChanged;

        public Func<Task> ExecutedHandler { get; private set; }

        public Func<bool> CanExecuteHandler { get; private set; }

        public AsyncCommand(Func<Task> executedHandler, Func<bool> canExecuteHandler = null)
        {
            ExecutedHandler = executedHandler;
            CanExecuteHandler = canExecuteHandler;
        }

        public async Task ExecuteAsync()
        {
            if (!CanExecute())
            {
                return;
            }

            await ExecutedHandler();
        }

        public bool CanExecute()
        {
            return CanExecuteHandler == null || CanExecuteHandler();
        }

        public void RaiseCanExecuteChanged()
        {
            if (CanExecuteChanged != null)
            {
                CanExecuteChanged(this, new System.EventArgs());
            }
        }

        bool ICommand.CanExecute(object parameter)
        {
            return CanExecute();
        }

        async void ICommand.Execute(object parameter)
        {
            await ExecuteAsync();
        }
    }

    public sealed class AsyncCommand<T> : ICommand
    {
        public event EventHandler CanExecuteChanged;

        public Func<T, Task> ExecutedHandler { get; private set; }

        public Func<bool> CanExecuteHandler { get; private set; }

        public AsyncCommand(Func<T, Task> executedHandler, Func<bool> canExecuteHandler = null)
        {
            ExecutedHandler = executedHandler;
            CanExecuteHandler = canExecuteHandler;
        }

        public async Task ExecuteAsync(T parameter)
        {
            if (!CanExecute())
            {
                return;
            }

            await ExecutedHandler(parameter);
        }

        public bool CanExecute()
        {
            return CanExecuteHandler == null || CanExecuteHandler();
        }

        public void RaiseCanExecuteChanged()
        {
            if (CanExecuteChanged != null)
            {
                CanExecuteChanged(this, new System.EventArgs());
            }
        }

        bool ICommand.CanExecute(object parameter)
        {
            return CanExecute();
        }

        async void ICommand.Execute(object parameter)
        {
            await ExecuteAsync((T) parameter);
        }
    }
}