﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;

namespace SfCalendarsample
{
    public class CalendarDatePickerViewModel : ViewModelBase
    {
        private AsyncCommand<DateTime> _dateSelectedCommand;

        public CalendarDatePickerViewModel()
        {
            _selectedDate = DateTime.Now;
        }

        public Task Initialize(object parameter)
        {
            ChangeMonthCommand = new RelayCommand(ChangeMonth);
            SelectTodayCommand = new RelayCommand(SelectToday);

            return Task.CompletedTask;
        }

        public RelayCommand ChangeMonthCommand { get; private set; }
        public RelayCommand SelectTodayCommand { get; private set; }

        private bool _monthArrowPressed;

        private DateTime _minFilterDate;

    


        public DateTime MinFilterDate
        {
            get { return _minFilterDate; }
            set { Set(ref _minFilterDate, value); }
        }

        private DateTime _maxFilterDate;

        public DateTime MaxFilterDate
        {
            get { return _maxFilterDate; }
            set { Set(ref _maxFilterDate, value); }
        }

        private object _selectedDate;

   
        public object SelectedDate
        {
            get { return _selectedDate; }
            set
            {
                if (value != null)
                {
                   
                        if (!_monthArrowPressed && !value.Equals(_selectedDate))
                        {
                            Set(ref _selectedDate, value);
                            RaisePropertyChanged(() => DisplayFormattedDate);
                            CalendarIsOpen = false;
                          

                        }
                  
                }
               
                _monthArrowPressed = false;
                
            }
        }

        private bool _calendarIsOpen;

        public bool CalendarIsOpen
        {
            get { return _calendarIsOpen; }
            set { Set(ref _calendarIsOpen, value); }
        }

        private DateTime _displayDate;

        public DateTime DisplayDate
        {
            get { return _displayDate; }
            set { Set(ref _displayDate, value); }
        }

        private string _displayFormat;

        public string DisplayFormattedDate
        {
            get { return SelectedDate.ToString(); }
        }

        public void SetInitialData(DateTime minDate,
            DateTime maxDate,
            DateTime initialDate,
            AsyncCommand<DateTime> dateSelectedCommand)
        {
            SetInitialData(minDate, maxDate, initialDate, dateSelectedCommand, string.Empty);
        }

        public void SetInitialData(DateTime minDate,
            DateTime maxDate,
            DateTime initialDate,
            AsyncCommand<DateTime> dateSelectedCommand,
            string displayFormat,
            double buttonWidth = 150)
        {
            _dateSelectedCommand = dateSelectedCommand;
            MinFilterDate = minDate;
            MaxFilterDate = maxDate;
            SelectedDate = DisplayDate = initialDate;
            _displayFormat = displayFormat;
            ButtonWidth = buttonWidth;
        }

        private void ChangeMonth()
        {
            _monthArrowPressed = true;
        }

        private void SelectToday()
        {
            SelectedDate = DisplayDate = DateTime.Today;
        }

        private double _buttonWidth;
        public double ButtonWidth
        {
            get { return _buttonWidth; }
            private set { Set(ref _buttonWidth, value); }
        }

        private bool _isHighlighted;
        public bool IsHighlighted
        {
            get { return _isHighlighted; }
            set { Set(ref _isHighlighted, value); }
        }

    
    }
}